/*
    aladinlink.c

    Downloads data from a UWATEC Aladin Dive Computer

    Copyright (C) 2002 Matthias Cramer, cramer@freestone.net
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <fcntl.h>
#include <sys/termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <strings.h>
#include <string.h>
#include "version.h"
#include "config.h"
#include "aladin.h"
              
/*
 *  Function prototypes
 */

unsigned char * progressbar(int length, int max, int curr);
int openlink(char * device);
unsigned char swapbyte(unsigned char);
static void byteswap(char *, int);

int main(int argc, char **argv) {
  
  int fd, fd2;
  fd_set rfds;
  struct timeval tv;
  int rec;
  int count;
  unsigned char buf[1];
  unsigned char * progress;

  struct s_config config;
  readconfig(&config);

  printf("aladinlink V%s %s\n", VERSION, COPY);

   if (argc == 2) {
     switch (argv[1][1]) {
       case 'h':
       case '?':
         printf("\nUsage: aladinlink [-r reserve] [-t beep] [-b breth] [-u units]\n\n");
         printf("       -s print status information\n");
         printf("       -r reserve in BAR (factory default 40)\n");
         printf("       -t beep on (1) off (0)\n");
         printf("       -b breth warning (-11 - 12 default 0)\n");
         printf("       -u units 0 = Metric, 1 = Imperial\n");
         exit(0);
     }
   }

   printf("\nPut your Aladin into LOG mode before starting this program.\n");
   printf("To abort press Ctrl-C !\n\n");

   fd2 = open("aladindump.raw",O_WRONLY | O_CREAT | O_TRUNC, 0644);

   fd = openlink(config.aladindevice);
  
   if (fd < 1) {
     printf("Error opening serial port!!");
     return(-1);
   }
   
   printf("Downloading data\n\n");
   progress = progressbar(70,BUFSIZE,0);
   
   FD_ZERO(&rfds);
   FD_SET(fd, &rfds);

   for (count=0; count < 3; count++) {
     rec = read(fd, buf, 1);
     if (buf[0] != 'U') {
       count = -1;
     }
   }
   rec = read(fd, buf, 1);
   if (buf[0] != 0) {
      printf("ERR: Start sequence not found in datastream.");
      return(-1);
   }

   count=0;

   while(count < BUFSIZE) {
     rec=0;
  
     tv.tv_sec=3;
     tv.tv_usec=0;
     
     if (!select(fd+1, &rfds, NULL, NULL, &tv)) {
       printf("\nError: transmission timeout, pleas check connection!\n");
       exit(1);
     }

     rec=read(fd,buf,1);
     if (rec>0) {
       count+=rec;

       if (count % 2 ==0) {
         progress = progressbar(70,BUFSIZE,count);
         printf("\r%s", progress);
       }

       byteswap(buf,1);
       write(fd2, &buf ,1);
     }
   }
   printf("\n\n");
   close(fd);
   close(fd2);
   
   return(0);
}

int openlink(char * device) {
  struct termios settings;
  int fd;
  int modelines=0;

   fd=open(device, O_RDWR|O_NOCTTY);
   if(fd==-1) {
     fprintf(stderr,"Device %s cannot be opened!\n",device);
     return(fd);
   }

   if(!isatty(fd)) {
     fprintf(stderr,"Device %s is not a terminal-device !\n",device);
     return(-1);
   }
   
   /*Set inputflags*/
   settings.c_iflag=   IGNBRK  /* Ignore Input breaks */
                     | IGNPAR; /* No Parity  */


   /*Set outputflags*/
   settings.c_oflag=0;         /* nothing special */

   /*Set controlflags*/
   settings.c_cflag= CS8     /* 8 bits per byte */
                   | CREAD   /* enable receiver */
                   | CLOCAL; /* Ignore modem status, line is local */

   /*Set local flags*/
   settings.c_lflag=0;         /* nothing special */  

   /*Set timeout*/
   settings.c_cc[VTIME]=20;  /* set timeout */

   /*Set minimal bytes to read*/
   settings.c_cc[VMIN]=1;
   
   /*Set baud rate*/
   if(cfsetspeed(&settings,B19200)!=0) {
     fprintf(stderr,"Failed to set baud rate on device %s !\n" ,device);
     return(-1);
   }

   /*Set parameters defined above to the serial device*/
   if(tcsetattr(fd,TCSANOW,&settings)==-1) {
     fprintf(stderr,"Failed to set terminal attributes on device %s !\n",device);
     return(-1);
   }

   /*Read io-controll parameters*/
   if(ioctl(fd,TIOCMGET,&modelines)==-1) {
     fprintf(stderr,"Error doing ioctl TIOCMGET on device %s!\n",device);
     return(-1);
   }

   /*Set RTS*/
   modelines &= ~TIOCM_RTS;
   if(ioctl(fd,TIOCMSET,&modelines)==-1) {
     fprintf(stderr,"Error doing ioctl TIOCMSET on device %s !\n",device); 
     return(-1);
   }
  return(fd);
}

unsigned char * progressbar(int length, int max, int curr) {
  static unsigned char buf[255];
  unsigned char bar[255];
  int i, proc, barlength;
  
  buf[0]=0;
  bar[0]=0;
  
  if (curr == 0) {
    proc = 0;
    barlength = 0;
  } else {
    proc = (int) (curr * 100.0 / max);
    barlength = (int) ((curr * 1.0/ max * 1.0) * (length-7.0));
  }

  for(i=1; i<=(length-7); i++) {
    if ( i<=barlength ) {
      strcat(bar, "#");
    } else {
      strcat(bar, "-");
    }
  }

  sprintf(buf, "|%s| %03d%%", bar, proc);
  return(buf);

}

static void byteswap(char * buf, int l) {
  int i;
  unsigned char x;
  
  for (i = 0; i < l; i++) {
    x  = (buf[i] & 0x01) << 7;
    x += (buf[i] & 0x02) << 5;
    x += (buf[i] & 0x04) << 3;
    x += (buf[i] & 0x08) << 1;
    x += (buf[i] & 0x10) >> 1;
    x += (buf[i] & 0x20) >> 3;
    x += (buf[i] & 0x40) >> 5;
    x += (buf[i] & 0x80) >> 7;
    buf[i] = x;
  }
}
